/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#ifndef BUFFEREDSETTINGS_H
#define BUFFEREDSETTINGS_H

#include <map>
#include <set>
#include <list>

#include "settings.hpp"

XERCES_CPP_NAMESPACE_USE

/**
 * The class BufferedSettings only reads the DOM-tree once
 * and caches all data it read. It also has some extra features
 * such as the insertCommandLineParameters function that is needed
 * by the main routine.
 */
class BufferedSettings : public Settings {
public:
     BufferedSettings(const std::string&);
     virtual ~BufferedSettings();
     virtual std::set<std::string> allParameters(const std::string&, const std::string&) const;
     SubParameters allSubParameters(const std::string&, const std::list<std::string>&) const;
     virtual void addParameter(const std::string&, const std::string&);
     bool insertCommandLineParameters(int, char**);
protected:
     virtual std::string parameterValue(const std::string&) const throw(ParamNotFoundException);
     virtual void buildBuffer(XERCES_CPP_NAMESPACE::DOMNode*, std::string);
     virtual void insert(const std::string&, const std::string&);
     typedef std::multimap<std::string, std::pair<std::string, int> > MapStrPStrInt;
     MapStrPStrInt settings;
};

#endif
