/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#ifndef CREATEWORDLIST_H
#define CREATEWORDLIST_H

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <string>
#include <map>
#include <vector>
#include <algorithm>
#include <cctype>

#include <boost/regex.hpp>

#include <xercesc/util/PlatformUtils.hpp>

#include "bufferedsettings.hpp"
#include "dynamic.hpp"

extern "C" {
#include <zlib.h>
}

XERCES_CPP_NAMESPACE_USE

typedef std::vector<Corpus::WordToken> TokenVector;
typedef std::multimap<std::string, std::pair<std::string, int> > WLT;

class StrIntMap {
public:
     StrIntMap() : count(0) { }

     int insert(const std::string& str) {
	  std::map<const std::string,int>::iterator ret = mymap.find(str);
	  if (ret != mymap.end()) {
        return ret->second;
	  }
	  else {
	       mymap.insert(std::make_pair(str, ++count));
	       return count;
	  }
     }

     typedef std::map<const std::string, int>::const_iterator mapiter;
     friend std::ostream& operator<<(std::ostream&, const StrIntMap&);
private:
     std::map<const std::string, int> mymap;
     int count;
};

std::ostream& operator<<(std::ostream& strm, const StrIntMap& sim) {
     for (StrIntMap::mapiter iter = sim.mymap.begin();
	  iter != sim.mymap.end();
	  ++iter) {
	  strm << iter->second << "\t" << iter->first << std::endl;
     }
     return strm;
}

/**
 * The createwordlist program is used to create the wordlist
 * that will be used by the program to create spelling errors.
 */
class CreateWordlist {
public:
     CreateWordlist(Settings &se);
private:
     void readFromFile(); 
     void transformData();
     void writeToFile();
     Settings &settings;
     std::string configDir;
     std::ifstream infile;
     gzFile outfile;
     std::ofstream tagfile;
     WLT wlt;
     WLT lwt;
     StrIntMap tagmap;
     TokenVector tokenvector;
};

#endif
