/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "simplesettings.hpp"

SimpleSettings::SimpleSettings(const std::string& filename) : Settings(filename) {

}

SimpleSettings::~SimpleSettings() {

}

std::set<std::string> SimpleSettings::allParameters(const std::string& path, const std::string& param) const {
     Separator separator("/");
     Tokenizer token(path, separator);
     std::set<std::string> retset;
     DOMNode *currentNode = parser->getDocument()->getDocumentElement();
     DOMNode *node = currentNode->getFirstChild();
     for (Tokenizer::iterator iter = token.begin(); iter != token.end(); ++iter) {
	  bool found = false;
	  for (node = currentNode->getFirstChild();
	       node != NULL;
	       node = node->getNextSibling()) {
	       if (node->getNodeType() == DOMNode::ELEMENT_NODE
		   && (XMLcmp(node->getNodeName(), XMLtr(iter->c_str()))==0)) {
		    currentNode = node;
		    found = true;
		    break;
	       }
	  }
	  if (!found) {
	       std::cerr << "Warning: setting " << path 
			 << "/" <<  param << " not found.\n";
	       return retset;
	  }
     }
     DOMNode *dn = node->getFirstChild();
     bool found = false;
     for (/*empty*/; dn != NULL; dn = dn->getNextSibling()) {
	  if (dn->getNodeType() == DOMNode::ELEMENT_NODE
	      && (XMLcmp(dn->getNodeName(), XMLtr(param.c_str()))==0)) {
	       for (DOMNode *sub = dn->getFirstChild(); dn != NULL; dn = dn->getNextSibling()) {
		    if (sub->getNodeType() == DOMNode::TEXT_NODE) {
			 found = true;
			 retset.insert(XMLtr(sub->getNodeValue()));
			 break;
		    }	  
	       }
	  }
     }
     if (!found) {
	  std::cerr << "Warning: setting " << path 
		    << "/" <<  param << " not found.\n";
     }
     return retset;
}

std::string SimpleSettings::parameterValue(const std::string& param) const throw(ParamNotFoundException) {
     Separator separator("/");
     Tokenizer token(param, separator);
     std::string retval;
     DOMNode *currentNode = parser->getDocument()->getDocumentElement();
     DOMNode *node;
     bool found = false;
     for (Tokenizer::iterator iter = token.begin(); iter != token.end(); ++iter) {
	  found = false;
	  for (node = currentNode->getFirstChild();
	       node != NULL;
	       node = node->getNextSibling()) {
	       if (node->getNodeType() == DOMNode::ELEMENT_NODE
		   && (XMLcmp(node->getNodeName(), XMLtr(iter->c_str()))==0)) {
		    currentNode = node;
		    found = true;
		    break;
	       }
	  }	  
	  if (!found) {
	       throw ParamNotFoundException();
	  }
     }
     for (DOMNode *dn = currentNode->getFirstChild(); dn != NULL; dn = dn->getNextSibling()) {
	  if (dn->getNodeType() == DOMNode::TEXT_NODE) {
	       return XMLtr(dn->getNodeValue());
	  }	  
     }
     throw ParamNotFoundException();
}
