/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#ifndef SPLITCOMPOUND_H
#define SPLITCOMPOUND_H

#include <iostream>
#include <string>
#include <set>
#include <list>

#include <boost/regex.hpp>

#include "wordlist.hpp"
#include "utils.hpp"
#include "dynamic.hpp"
#include "settings.hpp"
#include "wordtag.hpp"
#include "errorwriter.hpp"
#include "inputreader.hpp"

/**
 * A clasas used to split words in two parts if possible.
 */
class SplitCompound {
public:
     
     SplitCompound(Wordlist&, Settings&, ErrorWriter&, InputReader&);     
     bool apply(std::list<WordTag>&) const;     
     
private:
     
     unsigned int applyHelper(const std::string&, const int, const unsigned int,
			      const std::string&, const int,  
			      const unsigned int, const int, const int, WordTag&) const;
     std::pair<bool, std::pair<WordTag, WordTag> > apply(const WordTag&) const;
     bool tagAllowed(int) const;

     Wordlist &wordlist;
     Settings &settings;
     ErrorWriter &errorwriter;
     InputReader &inputreader;
     
     bool splitUnknownWords;
     unsigned int splitThreshold;
     unsigned int minWordLength;
     unsigned int minSplitWordLength;
     unsigned int wordLengthFactor;
     unsigned int inDictionaryFirstFactor;
     unsigned int inDictionarySecondFactor;
     unsigned int tagAllowedFactor;
     unsigned int tagMatchFirstFactor;
     unsigned int tagMatchSecondFactor;
     std::string reportSplitCompound;
     
     std::set<std::string> allowedClasses;
};

#endif
