/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#ifndef WORDLISTITERATOR_H
#define WORDLISTITERATOR_H

#include <iterator>

#include "wordlist.hpp"
#include "dynamic.hpp"

// fwd
class Wordlist;

class WordlistIterator 
     : public std::iterator<std::forward_iterator_tag, void, void, void, void> {
protected:
     Wordlist &wordlist;
     const Corpus::WordToken *token;
public:
     // Contructor
     WordlistIterator(Wordlist &wl, const Corpus::WordToken *tok) 
	  : wordlist(wl)
	  , token(tok) { }

     // Copy constructor
     WordlistIterator(const WordlistIterator& wli) 
	  : wordlist(wli.wordlist)
	  , token(wli.token) { }

     // Dereference
     const Corpus::WordToken& operator*() {
	  return *token;
     }

     // Member access
     const Corpus::WordToken* operator->() {
	  return token;
     }

     // Increment operations
     WordlistIterator& operator++() {
	  token = (Corpus::WordToken *)(token + token->size() + 1);
	  return *this;
     }
     WordlistIterator& operator++(int) {
	  token = (Corpus::WordToken *)(token + token->size() + 1);
	  return *this;
     }

     // Comparison
     bool operator==(const WordlistIterator &wli) const {
	  return token == wli.token;
     }
     bool operator!=(const WordlistIterator &wli) const {
	  return token != wli.token;
     }

};

#endif

